# megafier.py:  Dave Greene, 08/08/08, based on metafier.py.
# Converts selection into a Calcyman Megatick Metacell pattern. 

import os
import golly as g
from glife import *

selrect = g.getselrect()
if len(selrect) == 0: g.exit("There is no selection.")

# check that a layer is available for the metafied pattern;
# if metafied layer already exists then we'll use that
layername = "megafied"
metalayer = -1
for i in xrange(g.numlayers()):
   if g.getname(i) == layername:
      metalayer = i
      break
if metalayer < 0 and g.numlayers() == g.maxlayers():
   g.exit("You need to delete a layer.")

# get the current selection
slist = g.getcells(selrect)
selwidth = selrect[2]
selheight = selrect[3]

# create a 2D list of 0s and 1s representing entire selection
livecell = [[0 for y in xrange(selheight)] for x in xrange(selwidth)]
for i in xrange(0, len(slist), 2):
   livecell[slist[i] - selrect[0]][slist[i+1] - selrect[1]] = 1

# create metafied pattern in separate layer
if metalayer >= 0:
   g.setlayer(metalayer)         # switch to existing layer
else:
   metalayer = g.addlayer()      # create new layer

# set rule to Life
rule()

# load or generate the OFFcell tile:
# TODO:  change directory back to data directory after building the
#        OFF and ON tiles in this script, similar to metapixel.py
OFFcellFileName = os.path.join(g.appdir(),"Scripts") + "\megacell-OFF.rle"
ONcellFileName = os.path.join(g.appdir(), "Scripts") + "\megacell-ON.rle"
if os.access(OFFcellFileName, os.R_OK) and os.access(ONcellFileName, os.R_OK):
    g.show("Opening megapixel-OFF and megapixel-ON from saved pattern file.")
    OFFcell = pattern(g.transform(g.load(OFFcellFileName),0,0,1,0,0,1))
    ONcell = pattern(g.transform(g.load(ONcellFileName),0,0,1,0,0,1))

else:
    g.exit("Could not find megapixel-OFF.rle and/or megapixel-ON.rle")

g.new(layername)   
g.autoupdate(True)
g.setoption("hashing", False)       # qlife setcell is faster

for j in xrange(selheight):
   for i in xrange(selwidth):
      golly.show("Placing (" + str(i+1) + "," + str(j+1) + ") tile" +
                 " in a " + str(selwidth) + " by " + str(selheight) + " rectangle.")
      if livecell[i][j]:
         ONcell.put(16384 * (i - j), 16384 * (i + j))
      else:
         OFFcell.put(16384 * (i - j), 16384 * (i + j))
      g.fit()
   
g.show("")
g.setoption("hashing", True)        # no point running a metapattern without hashing
g.setoption("hyperspeed", False)    # avoid going too fast
g.setbase(8)
g.setstep(8)
g.step()                            # save start and populate hash tables
